@@ -2,11 +2,11 @@
 
 # Copyright (c) 2016 The OpenRC Authors.
 # See the Authors file at the top-level directory of this distribution and
-# https://github.com/OpenRC/openrc/blob/master/AUTHORS
+# https://github.com/OpenRC/openrc/blob/HEAD/AUTHORS
 #
 # This file is part of OpenRC. It is subject to the license terms in
 # the LICENSE file found in the top-level directory of this
-# distribution and at https://github.com/OpenRC/openrc/blob/master/LICENSE
+# distribution and at https://github.com/OpenRC/openrc/blob/HEAD/LICENSE
 # This file may not be copied, modified, propagated, or distributed
 #    except according to the terms contained in the LICENSE file.
 
@@ -36,6 +36,9 @@ supervise_start()
 		${respawn_period:+--respawn-period} $respawn_period \
 		${healthcheck_delay:+--healthcheck-delay} $healthcheck_delay \
 		${healthcheck_timer:+--healthcheck-timer} $healthcheck_timer \
+		${capabilities+--capabilities} "$capabilities" \
+		${secbits:+--secbits} "$secbits" \
+		${no_new_privs:+--no_new_privs} \
 		${command_user+--user} $command_user \
 		${umask+--umask} $umask \
 		${supervise_daemon_args:-${start_stop_daemon_args}} \
