@@ -6,19 +6,18 @@
 /*
  * Copyright (c) 2007-2015 The OpenRC Authors.
  * See the Authors file at the top-level directory of this distribution and
- * https://github.com/OpenRC/openrc/blob/master/AUTHORS
+ * https://github.com/OpenRC/openrc/blob/HEAD/AUTHORS
  *
  * This file is part of OpenRC. It is subject to the license terms in
  * the LICENSE file found in the top-level directory of this
- * distribution and at https://github.com/OpenRC/openrc/blob/master/LICENSE
+ * distribution and at https://github.com/OpenRC/openrc/blob/HEAD/LICENSE
  * This file may not be copied, modified, propagated, or distributed
  *    except according to the terms contained in the LICENSE file.
  */
 
-const char librc_copyright[] = "Copyright (c) 2007-2008 Roy Marples";
-
 #include "queue.h"
 #include "librc.h"
+#include <helpers.h>
 #ifdef __FreeBSD__
 #  include <sys/sysctl.h>
 #endif
@@ -49,6 +48,7 @@ static const rc_service_state_name_t rc_service_state_names[] = {
 	{ RC_SERVICE_HOTPLUGGED,  "hotplugged" },
 	{ RC_SERVICE_FAILED,      "failed" },
 	{ RC_SERVICE_SCHEDULED,   "scheduled"},
+	{ RC_SERVICE_CRASHED,     "crashed"},
 	{ 0, NULL}
 };
 
@@ -232,7 +232,7 @@ detect_prefix(const char *systype)
 }
 
 static const char *
-detect_container(const char *systype)
+detect_container(const char *systype _unused)
 {
 #ifdef __FreeBSD__
 	if (systype) {
@@ -296,7 +296,7 @@ detect_container(const char *systype)
 }
 
 static const char *
-detect_vm(const char *systype)
+detect_vm(const char *systype _unused)
 {
 #ifdef __NetBSD__
 	if (systype) {
@@ -349,7 +349,6 @@ rc_sys(void)
 
 	return sys;
 }
-librc_hidden_def(rc_sys)
 
 static const char *
 rc_parse_service_state(RC_SERVICE state)
@@ -424,20 +423,17 @@ rc_runlevel_starting(void)
 {
 	return exists(RC_STARTING);
 }
-librc_hidden_def(rc_runlevel_starting)
 
 bool
 rc_runlevel_stopping(void)
 {
 	return exists(RC_STOPPING);
 }
-librc_hidden_def(rc_runlevel_stopping)
 
 RC_STRINGLIST *rc_runlevel_list(void)
 {
 	return ls_dir(RC_RUNLEVELDIR, LS_DIR);
 }
-librc_hidden_def(rc_runlevel_list)
 
 char *
 rc_runlevel_get(void)
@@ -464,7 +460,6 @@ rc_runlevel_get(void)
 
 	return runlevel;
 }
-librc_hidden_def(rc_runlevel_get)
 
 bool
 rc_runlevel_set(const char *runlevel)
@@ -477,7 +472,6 @@ rc_runlevel_set(const char *runlevel)
 	fclose(fp);
 	return true;
 }
-librc_hidden_def(rc_runlevel_set)
 
 bool
 rc_runlevel_exists(const char *runlevel)
@@ -485,14 +479,14 @@ rc_runlevel_exists(const char *runlevel)
 	char path[PATH_MAX];
 	struct stat buf;
 
-	if (!runlevel || strcmp(runlevel, ".") == 0 || strcmp(runlevel, "..") == 0)
+	if (!runlevel || strcmp(runlevel, "") == 0 || strcmp(runlevel, ".") == 0 ||
+		strcmp(runlevel, "..") == 0)
 		return false;
 	snprintf(path, sizeof(path), "%s/%s", RC_RUNLEVELDIR, runlevel);
 	if (stat(path, &buf) == 0 && S_ISDIR(buf.st_mode))
 		return true;
 	return false;
 }
-librc_hidden_def(rc_runlevel_exists)
 
 bool
 rc_runlevel_stack(const char *dst, const char *src)
@@ -505,7 +499,6 @@ rc_runlevel_stack(const char *dst, const char *src)
 	snprintf(d, sizeof(s), "%s/%s/%s", RC_RUNLEVELDIR, dst, src);
 	return (symlink(s, d) == 0 ? true : false);
 }
-librc_hidden_def(rc_runlevel_stack)
 
 bool
 rc_runlevel_unstack(const char *dst, const char *src)
@@ -515,7 +508,6 @@ rc_runlevel_unstack(const char *dst, const char *src)
 	snprintf(path, sizeof(path), "%s/%s/%s", RC_RUNLEVELDIR, dst, src);
 	return (unlink(path) == 0 ? true : false);
 }
-librc_hidden_def(rc_runlevel_unstack)
 
 RC_STRINGLIST *
 rc_runlevel_stacks(const char *runlevel)
@@ -528,7 +520,6 @@ rc_runlevel_stacks(const char *runlevel)
 	rc_stringlist_free(ancestor_list);
 	return stack;
 }
-librc_hidden_def(rc_runlevel_stacks)
 
 /* Resolve a service name to its full path */
 char *
@@ -547,16 +538,16 @@ rc_service_resolve(const char *service)
 
 	/* First check started services */
 	snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s", "started", service);
-	if (lstat(file, &buf) || ! S_ISLNK(buf.st_mode)) {
+	if (lstat(file, &buf) || !S_ISLNK(buf.st_mode)) {
 		snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
 		    "inactive", service);
-		if (lstat(file, &buf) || ! S_ISLNK(buf.st_mode))
+		if (lstat(file, &buf) || !S_ISLNK(buf.st_mode))
 			*file = '\0';
 	}
 
 	if (*file) {
 		memset(buffer, 0, sizeof(buffer));
-		r = readlink(file, buffer, sizeof(buffer));
+		r = readlink(file, buffer, sizeof(buffer)-1);
 		if (r > 0)
 			return xstrdup(buffer);
 	}
@@ -582,7 +573,6 @@ rc_service_resolve(const char *service)
 
 	return NULL;
 }
-librc_hidden_def(rc_service_resolve)
 
 bool
 rc_service_exists(const char *service)
@@ -621,7 +611,6 @@ rc_service_exists(const char *service)
 	free(file);
 	return retval;
 }
-librc_hidden_def(rc_service_exists)
 
 #define OPTSTR \
 ". '%s'; echo $extra_commands $extra_started_commands $extra_stopped_commands"
@@ -663,7 +652,6 @@ rc_service_extra_commands(const char *service)
 	free(cmd);
 	return commands;
 }
-librc_hidden_def(rc_service_extra_commands)
 
 #define DESCSTR ". '%s'; echo \"${description%s%s}\""
 char *
@@ -693,7 +681,6 @@ rc_service_description(const char *service, const char *option)
 	free(cmd);
 	return desc;
 }
-librc_hidden_def(rc_service_description)
 
 bool
 rc_service_in_runlevel(const char *service, const char *runlevel)
@@ -704,7 +691,6 @@ rc_service_in_runlevel(const char *service, const char *runlevel)
 	    runlevel, basename_c(service));
 	return exists(file);
 }
-librc_hidden_def(rc_service_in_runlevel)
 
 bool
 rc_service_mark(const char *service, const RC_SERVICE state)
@@ -756,7 +742,7 @@ rc_service_mark(const char *service, const RC_SERVICE state)
 			s != RC_SERVICE_STOPPED &&
 			s != RC_SERVICE_HOTPLUGGED &&
 			s != RC_SERVICE_SCHEDULED) &&
-		    (! skip_wasinactive || s != RC_SERVICE_WASINACTIVE))
+		    (!skip_wasinactive || s != RC_SERVICE_WASINACTIVE))
 		{
 			snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
 			    rc_service_state_names[i].name, base);
@@ -824,7 +810,6 @@ rc_service_mark(const char *service, const RC_SERVICE state)
 	free(init);
 	return true;
 }
-librc_hidden_def(rc_service_mark)
 
 RC_SERVICE
 rc_service_state(const char *service)
@@ -847,6 +832,10 @@ rc_service_state(const char *service)
 		}
 	}
 
+	if (state & RC_SERVICE_STARTED) {
+		if (rc_service_daemons_crashed(service) && errno != EACCES)
+			state |= RC_SERVICE_CRASHED;
+	}
 	if (state & RC_SERVICE_STOPPED) {
 		dirs = ls_dir(RC_SVCDIR "/scheduled", 0);
 		TAILQ_FOREACH(dir, dirs, entries) {
@@ -863,7 +852,6 @@ rc_service_state(const char *service)
 
 	return state;
 }
-librc_hidden_def(rc_service_state)
 
 char *
 rc_service_value_get(const char *service, const char *option)
@@ -878,7 +866,6 @@ rc_service_value_get(const char *service, const char *option)
 
 	return buffer;
 }
-librc_hidden_def(rc_service_value_get)
 
 bool
 rc_service_value_set(const char *service, const char *option,
@@ -893,14 +880,16 @@ rc_service_value_set(const char *service, const char *option,
 		return false;
 
 	snprintf(p, sizeof(file) - (p - file), "/%s", option);
-	if (!(fp = fopen(file, "w")))
-		return false;
-	if (value)
+	if (value) {
+		if (!(fp = fopen(file, "w")))
+			return false;
 		fprintf(fp, "%s", value);
-	fclose(fp);
-	return true;
+		fclose(fp);
+	} else {
+		unlink(file);
+	}
+		return true;
 }
-librc_hidden_def(rc_service_value_set)
 
 
 bool
@@ -912,7 +901,7 @@ rc_service_schedule_start(const char *service, const char *service_to_start)
 	bool retval;
 
 	/* service may be a provided service, like net */
-	if (! service || ! rc_service_exists(service_to_start))
+	if (!service || !rc_service_exists(service_to_start))
 		return false;
 
 	p += snprintf(file, sizeof(file), RC_SVCDIR "/scheduled/%s",
@@ -927,7 +916,6 @@ rc_service_schedule_start(const char *service, const char *service_to_start)
 	free(init);
 	return retval;
 }
-librc_hidden_def(rc_service_schedule_start)
 
 bool
 rc_service_schedule_clear(const char *service)
@@ -940,7 +928,6 @@ rc_service_schedule_clear(const char *service)
 		return true;
 	return false;
 }
-librc_hidden_def(rc_service_schedule_clear)
 
 RC_STRINGLIST *
 rc_services_in_runlevel(const char *runlevel)
@@ -960,11 +947,11 @@ rc_services_in_runlevel(const char *runlevel)
 
 #ifdef RC_PKG_INITDIR
 		TAILQ_CONCAT(list, pkg, entries);
-		free(pkg);
+		rc_stringlist_free(pkg);
 #endif
 #ifdef RC_LOCAL_INITDIR
 		TAILQ_CONCAT(list, local, entries);
-		free(local);
+		rc_stringlist_free(local);
 #endif
 		return list;
 	}
@@ -978,7 +965,6 @@ rc_services_in_runlevel(const char *runlevel)
 		list = rc_stringlist_new();
 	return list;
 }
-librc_hidden_def(rc_services_in_runlevel)
 
 RC_STRINGLIST *
 rc_services_in_runlevel_stacked(const char *runlevel)
@@ -991,11 +977,11 @@ rc_services_in_runlevel_stacked(const char *runlevel)
 	TAILQ_FOREACH(stack, stacks, entries) {
 		sl = rc_services_in_runlevel(stack->value);
 		TAILQ_CONCAT(list, sl, entries);
-		free(sl);
+		rc_stringlist_free(sl);
 	}
+	rc_stringlist_free(stacks);
 	return list;
 }
-librc_hidden_def(rc_services_in_runlevel_stacked)
 
 RC_STRINGLIST *
 rc_services_in_state(RC_SERVICE state)
@@ -1015,7 +1001,7 @@ rc_services_in_state(RC_SERVICE state)
 
 	dirs = ls_dir(dir, 0);
 	list = rc_stringlist_new();
-	if (! dirs)
+	if (!dirs)
 		return list;
 
 	TAILQ_FOREACH(d, dirs, entries) {
@@ -1023,13 +1009,12 @@ rc_services_in_state(RC_SERVICE state)
 		services = ls_dir(dir, LS_INITD);
 		if (services) {
 			TAILQ_CONCAT(list, services, entries);
-			free(services);
+			rc_stringlist_free(services);
 		}
 	}
 	rc_stringlist_free(dirs);
 	return list;
 }
-librc_hidden_def(rc_services_in_state)
 
 bool
 rc_service_add(const char *runlevel, const char *service)
@@ -1038,7 +1023,6 @@ rc_service_add(const char *runlevel, const char *service)
 	char *init;
 	char file[PATH_MAX];
 	char path[MAXPATHLEN] = { '\0' };
-	char *p = NULL;
 	char binit[PATH_MAX];
 	char *i;
 
@@ -1059,8 +1043,7 @@ rc_service_add(const char *runlevel, const char *service)
 	/* We need to ensure that only things in /etc/init.d are added
 	 * to the boot runlevel */
 	if (strcmp(runlevel, RC_LEVEL_BOOT) == 0) {
-		p = realpath(dirname(init), path);
-		if (!*p) {
+		if (realpath(dirname(init), path) == NULL) {
 			free(init);
 			return false;
 		}
@@ -1077,7 +1060,6 @@ rc_service_add(const char *runlevel, const char *service)
 	free(init);
 	return retval;
 }
-librc_hidden_def(rc_service_add)
 
 bool
 rc_service_delete(const char *runlevel, const char *service)
@@ -1090,7 +1072,6 @@ rc_service_delete(const char *runlevel, const char *service)
 		return true;
 	return false;
 }
-librc_hidden_def(rc_service_delete)
 
 RC_STRINGLIST *
 rc_services_scheduled_by(const char *service)
@@ -1109,7 +1090,6 @@ rc_services_scheduled_by(const char *service)
 	rc_stringlist_free(dirs);
 	return list;
 }
-librc_hidden_def(rc_services_scheduled_by)
 
 RC_STRINGLIST *
 rc_services_scheduled(const char *service)
@@ -1120,4 +1100,3 @@ rc_services_scheduled(const char *service)
 	    basename_c(service));
 	return ls_dir(dir, LS_INITD);
 }
-librc_hidden_def(rc_services_scheduled)
